<?php
/**
 * Copyright (с) Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2022 All Rights Reserved
 *
 * Licensed under CLOUD LINUX LICENSE AGREEMENT
 * https://www.cloudlinux.com/legal/
 */

namespace CloudLinux\SmartAdvice\App\Service;

/**
 * Mail service
 */
class Mail {
	/**
	 * Panel user type string.
	 *
	 * @var string
	 */
	const USER_PANEL_TYPE = 'panel';

	/**
	 * WordPress user type string.
	 *
	 * @var string
	 */
	const USER_WP_TYPE = 'wp';

	/**
	 * Domain.
	 *
	 * @var string
	 */
	private $domain;

	/**
	 * Home url.
	 *
	 * @var string
	 */
	private $home;

	/**
	 * Lve url.
	 *
	 * @var string
	 */
	private $lve_url;

	/**
	 * Constructor.
	 *
	 * @param string  $domain site.
	 * @param string  $home url.
	 * @param ?string $lve_url url.
	 */
	public function __construct( $domain, $home, $lve_url = null ) {
		$this->domain  = $domain;
		$this->home    = $home;
		$this->lve_url = $lve_url;
		add_action( 'cl_smart_advice_notify', array( $this, 'notify' ), 10, 2 );
	}

	/**
	 * Domain.
	 *
	 * @return string
	 */
	public function domain() {
		return $this->domain;
	}

	/**
	 * Home.
	 *
	 * @return string
	 */
	public function home() {
		return $this->home;
	}

	/**
	 * Lve panel url.
	 *
	 * @return ?string
	 */
	public function lveUrl() {
		return $this->lve_url;
	}

	/**
	 * Notifying.
	 *
	 * @param int    $number_of_advices count.
	 * @param string $panel_user_emails emails.
	 *
	 * @return void
	 */
	public function notify( $number_of_advices, $panel_user_emails ) {
		do_action( 'cl_smart_advice_set_error_handler' );

		if ( $number_of_advices <= 0 ) {
			return;
		}

		$lve_url    = $this->lveUrl();
		$recipients = $this->recipients( $panel_user_emails );

		foreach ( $recipients as $type => $emails ) {
			$data = $this->data( $type, $number_of_advices, $lve_url );

			foreach ( $emails as $email ) {
				$this->send(
					$type,
					$email,
					$data['subject'],
					$data['body'],
					$data['headers']
				);
			}
		}

		do_action( 'cl_smart_advice_restore_error_handler' );
	}

	/**
	 * Merge WP users and Panel with duplicate filtering
	 *
	 * @param string $panel_user_emails List of emails.
	 *
	 * @return array
	 */
	public function recipients( $panel_user_emails ) {
		$admin_users  = get_users( array( 'role__in' => array( 'Administrator' ) ) );
		$admin_emails = array();
		foreach ( $admin_users as $user ) {
			if ( ! empty( $user->user_email ) ) {
				$admin_emails[] = $user->user_email;
			}
		}

		$panel_emails = explode( ',', $panel_user_emails );
		$panel_emails = array_map( 'trim', $panel_emails );
		$panel_emails = array_diff( $panel_emails, array( '' ) );
		$panel_emails = array_diff( $panel_emails, $admin_emails );

		return array(
			self::USER_PANEL_TYPE => $panel_emails,
			self::USER_WP_TYPE    => $admin_emails,
		);
	}

	/**
	 * Letter data
	 *
	 * @param string  $user_type type.
	 * @param int     $number_of_advices counter.
	 * @param ?string $panel_url url.
	 *
	 * @return array
	 */
	public function data( $user_type, $number_of_advices, $panel_url = '' ) {
		$attr = array(
			'{{ number_of_advices }}' => $number_of_advices,
			'{{ site_link }}'         => $this->home(),
			'{{ site_url }}'          => $this->domain(),
			'{{ panel_link }}'        => esc_attr( admin_url( 'options-general.php?page=' . CL_SMART_ADVICE_SLUG ) ),
		);

		if ( self::USER_PANEL_TYPE === $user_type && ! empty( $panel_url ) ) {
			$attr['{{ panel_link }}'] = $panel_url;
		}

		return array(
			'headers' => array( 'Content-Type: text/html; charset=UTF-8' ),
			'subject' => 'New performance Advice for ' . $attr['{{ site_url }}'],
			'body'    => $this->body( $attr ),
		);
	}

	/**
	 * Render body
	 *
	 * @param array $attr letter.
	 *
	 * @return ?string
	 */
	public function body( $attr ) {
		$path = CL_SMART_ADVICE_FOLDER_PATH . '/views/letter.php';
		if ( is_readable( $path ) ) {
			return strtr( file_get_contents( $path ), $attr ); // @phpcs:ignore WordPress.WP.AlternativeFunctions.file_get_contents_file_get_contents
		}

		return null;
	}

	/**
	 * Send emails.
	 *
	 * @param string $user_type Type of user.
	 * @param string $email Email.
	 * @param string $subject Letter subject.
	 * @param string $body Letter body.
	 * @param array  $headers Letter header.
	 *
	 * @return void
	 */
	public function send( $user_type, $email, $subject, $body, $headers ) {
		if ( self::USER_PANEL_TYPE === $user_type ) {
			add_filter( 'wp_mail_from', array( $this, 'panelSenderMail' ) );
			add_filter( 'wp_mail_from_name', array( $this, 'panelSenderName' ) );
		}
		wp_mail( $email, $subject, $body, $headers );
		remove_filter( 'wp_mail_from', array( $this, 'panelSenderMail' ) );
		remove_filter( 'wp_mail_from_name', array( $this, 'panelSenderName' ) );
	}

	/**
	 * A cPanel Sender Mailbox.
	 *
	 * @return string
	 */
	public function panelSenderMail() {
		return 'AccelerateWP@' . $this->domain();
	}

	/**
	 * A cPanel Sender Name.
	 *
	 * @return string
	 */
	public function panelSenderName() {
		return 'AccelerateWP';
	}
}
