# -*- coding: utf-8 -*-

# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENSE.TXT

import logging
import json
import os
import pwd

from typing import Any
from schema import Schema, SchemaError

from ..apiclient import get_client
from ..internal.exceptions import XRayAPIError
from .schemas import pullzone_info_schema
from ..internal.utils import timestamp

class AWPProvisionAPI:
    """Awp provision service handlers"""

    def __init__(self):
        self.logger = logging.getLogger('awp_provision_api')
        # initialize Adviser API client: awp provision uses same host as adviser
        awp_client_object = get_client('awp_provision')
        self.awp_client_api = awp_client_object()

    @staticmethod
    def _read_account_id(home_directory: str):
        account_auth_file = f'{home_directory}/.auth'
        if not os.path.isfile(account_auth_file):
            raise XRayAPIError('Auth file was not found!')
        with open(account_auth_file) as f:
            account_id = f.read().strip()
        return account_id

    def _account_id(self, account_id=None):
        if not account_id:
            return self._read_account_id(pwd.getpwuid(int(os.environ.get('XRAYEXEC_UID'))).pw_dir)
        return account_id

    @staticmethod
    def response(**kwargs) -> str:
        initial = {'result': 'success', 'timestamp': timestamp()}
        if kwargs:
            initial.update(kwargs)
        return json.dumps(dict(sorted(initial.items())))

    def _validate(self, data: Any, schema: Schema) -> Any:
        """Validate given data using given schema"""
        try:
            return schema.validate(data)
        except SchemaError as e:
            self.logger.error('Failed to validate API response: %s', data)
            msg = e.errors[-1] or e.autos[-1]
            raise XRayAPIError(f'Malformed API response: {msg}')

    def get_pullzone(self, account_id, domain, website):
        account_id = self._account_id(account_id)
        data = self.awp_client_api.get_create_pullzone(account_id, domain, website)
        response = self._validate(data=data,
                                  schema=pullzone_info_schema)
        return self.response(data=response)

    def remove_pullzone(self, account_id, domain, website):
        account_id = self._account_id(account_id)
        response = self.awp_client_api.remove_pullzone(account_id, domain, website)
        return self.response(data=response)

    def purge_cdn_cache(self, account_id, domain, website):
        account_id = self._account_id(account_id)
        response = self.awp_client_api.purge_cdn_cache(account_id, domain, website)
        return self.response(data=response)
